#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Cli = void 0;
const commander_1 = __importDefault(require("commander"));
const core_1 = require("./../core");
const config_1 = require("./../core/config");
const enums_1 = require("./enums");
const chalk_1 = __importDefault(require("chalk"));
const utils_1 = require("./utils");
const name = 'react-i18next-lint';
// tslint:disable-next-line:no-any
const docs = {
    name,
    usage: '[options]',
    description: 'Simple CLI tools for check `react-i18next` keys in app',
    examples: `

Examples:

    $ ${name} -p ${config_1.config.defaultValues.projectPath} -l ${config_1.config.defaultValues.languagesPath}
    $ ${name} -p ${config_1.config.defaultValues.projectPath} -z ${core_1.ErrorTypes.disable} -v ${core_1.ErrorTypes.error}
    $ ${name} -p ${config_1.config.defaultValues.projectPath} -i ./src/assets/i18n/EN-us.json, ./src/app/app.*.{json}

`
};
class Cli {
    constructor(options) {
        this.cliClient = commander_1.default;
        this.cliOptions = [];
        this.cliOptions = options;
    }
    static run(options) {
        const cli = new Cli(options);
        cli.init();
        cli.parse();
        cli.runCli();
    }
    init(options = this.cliOptions) {
        options.forEach((option) => {
            const optionFlag = option.getFlag();
            const optionDescription = option.getDescription();
            const optionDefaultValue = option.default;
            this.cliClient.option(optionFlag, optionDescription, optionDefaultValue);
        });
        // tslint:disable-next-line:no-any
        const packageJson = (0, utils_1.parseJsonFile)((0, utils_1.getPackageJsonPath)());
        this.cliClient.version(packageJson.version);
        this.cliClient
            .name(docs.name)
            .usage(docs.usage)
            .description(docs.description)
            .on(`--${enums_1.OptionsLongNames.help}`, () => {
            // tslint:disable-next-line:no-console
            console.log(docs.examples);
        });
    }
    runCli() {
        try {
            // tslint:disable-next-line:no-any
            const options = this.cliClient.config ? (0, utils_1.parseJsonFile)(this.cliClient.config) : this.cliClient;
            const projectPath = options.project;
            const languagePath = options.languages;
            const tsConfigPath = options.tsConfigPath;
            let deepSearch;
            let optionIgnore;
            let optionMisprint;
            let optionEmptyKey;
            let optionViewsRule;
            let optionMaxWarning;
            let optionZombiesRule;
            let optionIgnoredKeys;
            let optionMisprintCoefficient;
            let optionIgnoredMisprintKeys;
            let optionCustomRegExpToFindKeys;
            if (!!options.rules) {
                deepSearch = options.rules.deepSearch;
                optionIgnore = options.rules.ignore;
                optionMisprint = options.rules.misprintKeys;
                optionEmptyKey = options.rules.emptyKeys;
                optionViewsRule = options.rules.keysOnViews;
                optionMaxWarning = options.rules.maxWarning;
                optionZombiesRule = options.rules.zombieKeys;
                optionIgnoredKeys = options.rules.ignoredKeys;
                optionMisprintCoefficient = options.rules.misprintCoefficient;
                optionIgnoredMisprintKeys = options.rules.ignoredMisprintKeys;
                optionCustomRegExpToFindKeys = options.rules.customRegExpToFindKeys;
            }
            else {
                deepSearch = options.deepSearch;
                optionIgnore = options.ignore;
                optionMisprint = options.misprintKeys;
                optionEmptyKey = options.emptyKeys;
                optionViewsRule = options.keysOnViews;
                optionMaxWarning = options.maxWarning;
                optionZombiesRule = options.zombieKeys;
                optionIgnoredKeys = options.ignoredKeys;
                optionMisprintCoefficient = options.misprintCoefficient;
                optionIgnoredMisprintKeys = options.ignoredMisprintKeys;
                optionCustomRegExpToFindKeys = options.customRegExpToFindKeys;
            }
            this.printCurrentVersion();
            if (options.project && options.languages) {
                this.runLint(projectPath, languagePath, optionZombiesRule, optionViewsRule, optionIgnore, optionMaxWarning, optionMisprint, optionEmptyKey, deepSearch, optionMisprintCoefficient, optionIgnoredKeys, optionIgnoredMisprintKeys, optionCustomRegExpToFindKeys, tsConfigPath);
            }
            else {
                const cliHasError = this.validate();
                if (cliHasError) {
                    process.exit(core_1.StatusCodes.crash);
                }
                else {
                    this.cliClient.help();
                }
            }
        }
        catch (error) {
            // tslint:disable-next-line: no-console
            console.error(error);
            process.exitCode = core_1.StatusCodes.crash;
        }
        finally {
            process.exit();
        }
    }
    parse() {
        this.cliClient.parse(process.argv);
    }
    validate() {
        const requiredOptions = this.cliOptions.filter((option) => option.required);
        const missingRequiredOption = requiredOptions.reduce((accum, option) => {
            if (!this.cliClient[String(option.longName)]) {
                accum = false;
                // tslint:disable-next-line: no-console
                console.error(`Missing required argument: ${option.getFlag()}`);
            }
            return accum;
        }, false);
        return missingRequiredOption;
    }
    runLint(project, languages, zombies, views, ignore, maxWarning = 1, misprint, emptyKeys, deepSearch, misprintCoefficient = 0.9, ignoredKeys = [], ignoredMisprintKeys = [], customRegExpToFindKeys = [], tsConfigPath) {
        const errorConfig = {
            misprintKeys: misprint || core_1.ErrorTypes.disable,
            deepSearch: deepSearch || core_1.ToggleRule.disable,
            zombieKeys: zombies || core_1.ErrorTypes.warning,
            emptyKeys: emptyKeys || core_1.ErrorTypes.warning,
            keysOnViews: views || core_1.ErrorTypes.error,
            maxWarning,
            ignoredKeys,
            ignoredMisprintKeys,
            misprintCoefficient,
            customRegExpToFindKeys,
        };
        const validationModel = new core_1.ReactI18nextLint(project, languages, ignore, errorConfig, tsConfigPath);
        const resultCliModel = validationModel.lint(maxWarning);
        const resultModel = resultCliModel.getResultModel();
        resultModel.printResult();
        resultModel.printSummery();
        process.exitCode = resultCliModel.exitCode();
        if (resultModel.hasError) {
            throw new core_1.FatalErrorModel(chalk_1.default.red(resultModel.message));
        }
    }
    printCurrentVersion() {
        // tslint:disable-next-line:no-any
        const packageJson = (0, utils_1.parseJsonFile)((0, utils_1.getPackageJsonPath)());
        // tslint:disable-next-line:no-console
        console.log(`Current version: ${packageJson.version}`);
    }
}
exports.Cli = Cli;
//# sourceMappingURL=cli.js.map